@@ -1,13 +1,16 @@
 import cPickle
 import os.path
+import sys
 from collections import defaultdict
 
+from pyparsing import ParseBaseException
+
 from scss import SORTING
 from scss.base import Node, Empty, ParseNode, ContentNode, IncludeNode
 from scss.control import Variable, Expression, Function, Mixin, Include, MixinParam, Extend, Variables, Option, FunctionDefinition, FunctionReturn, If, For, SepValString
 from scss.function import warn, _nest
 from scss.grammar import *
-from scss.value import NumberValue, StringValue, ColorValue, QuotedStringValue
+from scss.value import NumberValue, StringValue, ColorValue, QuotedStringValue, PointValue
 
 
 class Comment(Node):
@@ -202,6 +205,7 @@ class Stylesheet(object):
         NUMBER_VALUE.setParseAction(NumberValue)
         IDENT.setParseAction(StringValue)
         PATH.setParseAction(StringValue)
+        POINT.setParseAction(PointValue)
         COLOR_VALUE.setParseAction(ColorValue)
         quotedString.setParseAction(QuotedStringValue)
         EXPRESSION.setParseAction(Expression)
@@ -282,8 +286,15 @@ class Stylesheet(object):
         """ Scan scss from string and return nodes.
         """
         assert isinstance(src, basestring)
-        nodes = STYLESHEET.parseString(src)
-        return nodes
+        try:
+            nodes = STYLESHEET.parseString(src, parseAll=True)
+            return nodes
+        except ParseBaseException:
+            err = sys.exc_info()[1]
+            print >> sys.stderr, err.line
+            print >> sys.stderr, " "*(err.column-1) + "^"
+            print >> sys.stderr, err
+            sys.exit(1)
 
     def parse(self, nodes):
         map(lambda n: n.parse(self) if isinstance(n, Node) else None, nodes)
